<?php
/**
 * DDNotesBehavior class file 
 *
 * @author Joachim Werner <joachim.werner@diggin-data.de>
 */

Yii::import('application.extensions.ddnotes.DDNote');

/**
 * DDNotesBehavior 
 * 
 * @uses CActiveRecordBehavior
 * @copyright 2011 Joachim Werner <joachim.werner@diggin-data.de>
 * @author Joachim Werner <joachim.werner@diggin-data.de>
 */
class DDNotesBehavior extends CActiveRecordBehavior
{
    // {{{ *** Members ***
    /**
     * @var string ID of div (container) where to place note added success message
     * @access public
     */
    public $addSuccessDivID = 'noteSubmitted';
    public $showAddNoteButton = false;
    /**
     * @var string Log Category for debug/trace messages
     */
    const NB_LOG_CATEGORY='application.components.behaviors.NotesBehavior';
    // }}} 
    // {{{ *** Methods ***
    // {{{ attach
    public function attach($owner)
    {
	Yii::trace(__CLASS__.'.'.__FUNCTION__,self::NB_LOG_CATEGORY);
        parent::attach($owner);

        $owner->metaData->addRelation( 'notes', array( CActiveRecord::HAS_MANY, 'DDNote','foreignKey','condition'=>"notes.tableName='".$owner->tableName()."'",'order'=>'notes.tsCreated DESC' ) );
        $owner->metaData->addRelation( 'notesCount', array( CActiveRecord::STAT, 'DDNote','foreignKey', 'condition'=>"tableName='".$owner->tableName()."'") );
        
    } // }}} 
    // {{{ events
    /**
     * 
     * @see base/CBehavior::events()
     */
    public function events()
    {
        Yii::trace(__CLASS__.'.'.__FUNCTION__,self::NB_LOG_CATEGORY);
        $ev=array(
            'onAfterSave'   => 'afterSave',
            'onAfterDelete' => 'afterDelete',
        );
        return $ev;
    } // }}} 
    // {{{ afterSave
    public function afterSave($event)
    {    
        Yii::trace(__CLASS__.'.'.__FUNCTION__,self::NB_LOG_CATEGORY);
        if(isset($_POST['DDNote']))
        {
            $note=new DDNote;
            $note->attributes=$_POST['DDNote'];
            $note->tableName = $this->getTableNamePrepared();
            $note->foreignKey=$this->owner->getAttribute($this->owner->tableSchema->primaryKey);
            $note->tsCreated = mktime();
            $note->userIdCreated = Yii::app()->user->id;
            if($note->save())
            {
                Yii::app()->user->setFlash($this->addSuccessDivID,Yii::t('ddnotes','Your note has been added.'));
                //$this->refresh();
            }
        }
    } // }}} 
    // {{{ afterDelete
    public function afterDelete($event)
    {    
        Yii::trace(__CLASS__.'.'.__FUNCTION__,self::NB_LOG_CATEGORY);
        $sql = sprintf(
            "DELETE FROM %s WHERE tableName=:tableName AND foreignKey=:foreignKey",
            DDNote::model()->tableSchema->name
        );
        $command = Yii::app()->db->createCommand($sql);
        $command->bindValue(":tableName",$this->getTableNamePrepared(),PDO::PARAM_STR);
        $command->bindValue(":foreignKey",$this->owner->getAttribute($this->owner->tableSchema->primaryKey),PDO::PARAM_INT);
        $command->execute();
    } // }}} 
    // {{{ getNote
    /**
     * Creates a new note.
     * This method attempts to create a new note based on the user input.
     * If the note is successfully created, the browser will be redirected
     * to show the created note.
     * @param MyModel Model the model that the new note belongs to
     * @return Note the note instance
     */
    protected function getNote()
    {
        $note=new DDNote;
        if(isset($_POST['ajax']))
        {
            echo CActiveForm::validate($note);
            Yii::app()->end();
        }
        return $note;
    } // }}} 
    // {{{ addNote
    /**
     * Adds a new note to this model.
     * This method will set tableName and foreignKey of the note accordingly.
     * @param Note the note to be added
     * @return boolean whether the note is saved successfully
     */
    public function addNote($note)
    {
        $note->tableName = $this->getTableNamePrepared();
        $note->foreignKey=$this->owner->getAttribute($this->owner->tableSchema->primaryKey);
        return $note->save();
    } // }}} 
    // {{{ getTableNamePrepared
    /**
     * Returns the table name, having '{{...}}' replaced with DB table prefix 
     * 
     * @param string $tableName If null, current model's table name will be used
     * @access public
     * @return string Prepared table name
     */
    public function getTableNamePrepared($tableName=null)
    {
        if($tableName==null)
            $tableName = $this->owner->tableName();
        if(preg_match("/^{{(.*)}}$/", $tableName,$matches)) {
            return Yii::app()->db->tablePrefix.$matches[1];
        } else {
            return $tableName;
        }
    } // }}} 
    // }}} End Methods
}

/* vim:set ai sw=4 sts=4 et fdm=marker fdc=4: */ 
?>
